package net.sf.doolin.bus;

/**
 * Defines an interface for logging events that occur in a {@link Bus}.
 * 
 * @author Damien Coraboeuf
 * 
 */
public interface BusLog {

	/**
	 * This method is called when a subscriber is successfully registered.
	 * 
	 * @param subscriber
	 *            Registered subscriber.
	 */
	void registrationOk(Subscriber<?> subscriber);

	/**
	 * This method is called when a subscriber is unregistered.
	 * 
	 * @param subscriber
	 *            Unregistered subscriber.
	 */
	void unregistration(Subscriber<?> subscriber);

	/**
	 * This method is called when a message is received by the bus.
	 * 
	 * @param message
	 *            Received message
	 */
	void messageReceived(Object message);

	/**
	 * This method is called when no subscription was found to handle the
	 * message
	 * 
	 * @param message
	 *            Received message
	 */
	void messageNoSubscription(Object message);

	/**
	 * This method is called when a message is about to be handled by a
	 * subscriber.
	 * 
	 * @param <M>
	 *            Type of the message
	 * @param message
	 *            Received message
	 * @param subscriber
	 *            Subscriber about to handle the message
	 */
	<M> void messageAccepted(M message, Subscriber<M> subscriber);

	/**
	 * This method is called when the processing of a message by a subscriber
	 * has resulted in an error.
	 * 
	 * @param <M>
	 *            Type of the message
	 * @param message
	 *            Received message
	 * @param subscriber
	 *            Subscriber which handled the message
	 * @param ex
	 *            Resulting error
	 */
	<M> void messageWithError(M message, Subscriber<M> subscriber, Exception ex);

}
