package net.sf.doolin.bus;

/**
 * Defines a bus that will handle the messages synchronously.
 */
public interface IBus {

    /**
     * Publishes a message on this bus and waits for all subscribers to have completed
     * the treatment of this message.
     *
     * @param message Message which is published.
     */
    void publish(Object message);

    /**
     * Subscribes to this bus.
     *
     * @param subscriber Details of the subscription.
     * @param <M>        Type of message
     */
    <M> void subscribe(Subscriber<M> subscriber);

    /**
     * Unsubscribes to this bus.
     *
     * @param subscriber Details of the subscription.
     * @param <M>        Type of message
     */
    <M> void unsubscribe(Subscriber<M> subscriber);

    /**
     * Clears all subscriptions from the bus, use with care!
     */
    void clear();

    /**
     * Gets information about the bus
     */
    IBusInfo getBusInfo();

    /**
     * Gets the log associated with this bus
     */
	BusLog getLog();

	/**
	 * Sets the log associated with this bus
	 */
	void setLog(BusLog log);
}
