package net.sf.doolin.bus;

/**
 * A {@link SubscriberValidator} defines the life-cycle of a {@link Subscriber}
 * in the {@link Bus}.
 * <p/>
 * Whenever the bus receives a new messages, all potential {@link Subscriber}s
 * instances are checked with their associated {@link SubscriberValidator} in
 * order to see if the subscription is still valid at this instant.
 * <p/>
 * It is the responsibility of the implementation of this interface to decide
 * when a subscription is no longer valid.
 *
 * @author Damien Coraboeuf
 */
public interface SubscriberValidator {

    /**
     * Gets a description for this validator
     */
    String getValidatorDescription();

    /**
     * Checks if the subscription is still valid.
     *
     * @return <code>true</code> if the subscription is still valid,
     *         <code>false</code> otherwise.
     */
    boolean isValid();

}
