package net.sf.doolin.bus.support;

import net.sf.doolin.bus.Subscriber;
import net.sf.doolin.bus.SubscriberValidator;
import org.apache.commons.lang.Validate;

/**
 * Abstract subscriber that relies on an external {@link SubscriberValidator}
 * for its validation.
 *
 * @param <M> Type of message for this subscriber
 * @author Damien Coraboeuf
 */
public abstract class AbstractSubscriber<M> implements Subscriber<M> {

    private final Class<M> messageClass;
    private final SubscriberValidator validator;

    /**
     * Builds a subscriber.
     *
     * @param validator Validator to use for this subscriber.
     */
    public AbstractSubscriber(Class<M> messageClass, SubscriberValidator validator) {
        Validate.notNull(validator, "Validator must not be null");
        Validate.notNull(messageClass, "Message class must not be null");
        this.messageClass = messageClass;
        this.validator = validator;
    }

    @Override
    public Class<M> getMessageClass() {
        return messageClass;
    }

    @Override
    public String getValidatorDescription() {
        return this.validator.getValidatorDescription();
    }

    @Override
    public boolean isValid() {
        return this.validator.isValid();
    }

    @Override
    public String toString() {
        return String.format("trigger=[%s],executor=[%s],validator=[%s]", getTriggerDescription(), getExecutionDescription(), getValidatorDescription());
    }
}
