package net.sf.doolin.bus.support;

import net.sf.doolin.bus.SubscriberValidator;
import net.sf.doolin.util.Utils;

/**
 * This subscriber listens to property changes on a particular bean.
 *
 * @author Damien Coraboeuf
 */
public class PropertyChangeSubscriber extends
        AbstractSubscriber<PropertyChangeMessage> {

    private final String executionDescription;
    private final Object bean;
    private final String propertyName;
    private final PropertyChangeListener propertyChangeListener;

    /**
     * Constructor
     *
     * @param executionDescription   Description of the action that is performed by this subscriber
     * @param validator              Subscription validator
     * @param bean                   Bean to listen to
     * @param propertyName           Property path to listen to or <code>null</code> if all
     *                               properties must be listened to
     * @param propertyChangeListener Listener for the changes
     */
    public PropertyChangeSubscriber(String executionDescription,
                                    SubscriberValidator validator, Object bean, String propertyName,
                                    PropertyChangeListener propertyChangeListener) {
        super(PropertyChangeMessage.class, validator);
        this.executionDescription = executionDescription;
        this.bean = bean;
        this.propertyName = propertyName;
        this.propertyChangeListener = propertyChangeListener;
    }

    @Override
    public boolean accept(PropertyChangeMessage message) {
        return message.sameOrigin(bean)
                && (this.propertyName == null || this.propertyName
                .equals(message.getPropertyName()));
    }

    @Override
    public String getExecutionDescription() {
        return this.executionDescription;
    }

    @Override
    public String getTriggerDescription() {
        String p = this.propertyName != null ? this.propertyName : "any";
        String b = this.bean != null ? Utils.simpleToString(this.bean, "")
                : "any";
        String t = this.bean != null ? this.bean.getClass().getSimpleName()
                : "";
        return String.format("%s{%s}.%s", b, t, p);
    }

    @Override
    public void receive(PropertyChangeMessage message) {
        propertyChangeListener.onChange(message);
    }

}
