package net.sf.doolin.bus.support;

import net.sf.doolin.bus.Subscriber;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Message processor registration
 *
 * @param <M> Message type
 */
public class Subscription<M> {
    /**
     * Logger
     */
    private final static Logger log = LoggerFactory
            .getLogger(Subscription.class);

    /**
     * Associated message processor
     */
    private final Subscriber<M> subscriber;

    /**
     * Registration time
     */
    private final long registrationTime;

    /**
     * Constructor
     *
     * @param subscriber Associated subscriber
     */
    public Subscription(Subscriber<M> subscriber) {
        this.subscriber = subscriber;
        this.registrationTime = System.currentTimeMillis();
    }

    /**
     * Checks if this subscription accepts a message
     *
     * @param message Message to test
     * @return Result of the test
     */
    public boolean accept(M message) {
        return this.subscriber.accept(message);
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    @Override
    public boolean equals(Object o) {
        Subscription<?> registration = (Subscription<?>) o;
        boolean ok = this.subscriber == registration.subscriber;
        if (log.isTraceEnabled()) {
            log.trace("equals " + ok + " for " + o + " and " + this);
        }
        return ok;
    }

    /**
     * Returns the registration date
     *
     * @return Registration date
     * @see System#currentTimeMillis()
     */
    public long getRegistrationTime() {
        return this.registrationTime;
    }

    /**
     * @return Subscriber
     */
    public Subscriber<M> getSubscriber() {
        return this.subscriber;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return this.subscriber.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return this.subscriber.toString();
    }
}
