package net.sf.doolin.bus.support;

import net.sf.doolin.bus.ISubscriptionClassInfo;
import net.sf.doolin.bus.ISubscriptionInfo;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;

import java.util.ArrayList;
import java.util.List;

public class SubscriptionClassInfo implements ISubscriptionClassInfo {

    private final String messageClassName;
    private final List<ISubscriptionInfo> subscriptionInfos = new ArrayList<ISubscriptionInfo>();

    public SubscriptionClassInfo(Class<?> messageClass) {
        Validate.notNull(messageClass, "The message class is required");
        // Gets the class name
        String name = messageClass.getName();
        // Filters the class name
        if (name.contains("$")) {
            name = StringUtils.substringBefore(name, "$");
        }
        messageClassName = StringUtils.substringAfterLast(name, ".");
    }

    @Override
    public String getMessageClassName() {
        return messageClassName;
    }

    @Override
    public List<ISubscriptionInfo> getSubscriptionInfos() {
        return subscriptionInfos;
    }

    public void addSubscriptionInfo(Subscription<?> subscription) {
        SubscriptionInfo info = new SubscriptionInfo(subscription);
        subscriptionInfos.add(info);
    }
}
