package net.sf.doolin.bus.support;

import java.lang.ref.WeakReference;

import net.sf.doolin.bus.ISubscriptionInfo;
import net.sf.doolin.bus.Subscriber;

import org.apache.commons.lang.StringUtils;

public class SubscriptionInfo implements ISubscriptionInfo {

    private final String description;
    private final String trigger;
    private final String execution;
    private final String validator;
    private final String type;
    private final long registrationTime;
    
    private final WeakReference<Subscriber<?>> subscriberRef;

    public SubscriptionInfo(Subscription<?> subscription) {
        this.registrationTime = subscription.getRegistrationTime();

        Subscriber<?> subscriber = subscription.getSubscriber();

        String className = subscriber.getClass().getName();
        if (className.contains("$")) {
            className = StringUtils.substringBefore(className, "$");
        }
        type = StringUtils.substringAfterLast(className, ".");

        description = subscriber.toString();
        trigger = subscriber.getTriggerDescription();
        execution = subscriber.getExecutionDescription();
        validator = subscriber.getValidatorDescription();
        
        subscriberRef = new WeakReference<Subscriber<?>>(subscriber);
    }

    @Override
    public String getDescription() {
        return description;
    }

    @Override
    public String getType() {
        return type;
    }

    @Override
    public long getRegistrationTime() {
        return registrationTime;
    }
   
    @Override
    public String getExecution() {
    	return execution;
    }
    
    @Override
    public String getTrigger() {
    	return trigger;
    }
    
    @Override
    public String getValidator() {
    	return validator;
    }
    
    @Override
    public boolean isValid() {
    	Subscriber<?> subscriber = subscriberRef.get();
    	if (subscriber != null) {
    		return subscriber.isValid();
    	} else {
    		return false;
    	}
    }
    
}
