package net.sf.doolin.bus.support;

import com.google.common.collect.ImmutableList;
import net.sf.doolin.bus.Subscriber;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class Subscriptions<M> {

    private final Lock lock = new ReentrantLock();

    private final List<Subscription<M>> subscriptions = new ArrayList<Subscription<M>>();

    public List<Subscription<M>> getSubscriptions() {
        lock.lock();
        try {
            return ImmutableList.copyOf(subscriptions);
        } finally {
            lock.unlock();
        }
    }

    public void add(Subscriber<M> subscriber) {
        lock.lock();
        try {
            subscriptions.add(new Subscription<M>(subscriber));
        } finally {
            lock.unlock();
        }
    }

    public void remove(Subscriber<M> subscriber) {
        lock.lock();
        try {
            Iterator<Subscription<M>> i = subscriptions.iterator();
            while (i.hasNext()) {
                Subscription<M> subscription = i.next();
                if (subscription.getSubscriber() == subscriber) {
                    i.remove();
                }
            }
        } finally {
            lock.unlock();
        }
    }

    public int getSubscriberCount() {
        lock.lock();
        try {
            return subscriptions.size();
        } finally {
            lock.unlock();
        }
    }

    public void clear() {
        lock.lock();
        try {
            Iterator<Subscription<M>> i = subscriptions.iterator();
            while (i.hasNext()) {
                Subscription<M> subscription = i.next();
                if (!subscription.getSubscriber().isValid()) {
                    i.remove();
                }
            }
        } finally {
            lock.unlock();
        }
    }
}
